/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.context;

import org.apache.log4j.Logger;

/**
 *  Hold the context for an export(ing).
 *
 */
public final class XMLExportContext {

    private static final Logger log = Logger.getLogger(XMLExportContext.class);

    /**
     * Root stem for the XML entity identifier URI.
     * Includes terminal slash.
     */
    private static String uriStem;

    /**
     * Directory path for the location of the export file.
     * Includes terminal slash.
     */
    private static String fileDirPath;

    /**
     * Encoding scheme for the export files.
     * Defaults to XML.
     */
    private static String scheme;

    /**
     * Version level of the scheme definition for the export files.
     * Defaults to latest released version.
     */
    private static String version;

    /**
     * Extension level to export, of efrbr or vfrbr.
     * Defaults to efrbr.
     */
    private static String level;

    /*
     *  Default initializations.
     */
    static {
        uriStem = "http://vfrbr.info/";

        fileDirPath = "/usr/local/vfrbr/exports/xml/1.1/";

        scheme = "XML";

        version = "1.1";

        level = "efrbr";
    }

    /**
     * Non-instantiable singleton.
     */
    private XMLExportContext() {
    }

    /**
     *  Make a URI for XML FRBR entity reference.
     * @param entityType the type of the entity, like Work.
     * @param entityId the int id for the entity.
     * @return String of the URI.
     */
    public static String makeURI(final String entityType,
                          final int entityId) {

        return getUriStem() + entityType + "/" + entityId;
    }

    /**
     * Root stem for the XML entity identifier URI.
     * Includes terminal slash.
     * @return the uriStem
     */
    public static String getUriStem() {
        return uriStem;
    }

    /**
     * Root stem for the XML entity identifier URI.
     * Includes terminal slash.
     * @param uriStem the uriStem to set
     */
    public static void setUriStem(String uriStem) {
        XMLExportContext.uriStem = uriStem;
    }

    /**
     * Directory path for the location of the export file.
     * Includes terminal slash.
     * @return the fileDirPath
     */
    public static String getFileDirPath() {
        return fileDirPath;
    }

    /**
     * Directory path for the location of the export file.
     * Includes terminal slash.
     * @param fileDirPath the fileDirPath to set
     */
    public static void setFileDirPath(String fileDirPath) {
        XMLExportContext.fileDirPath = fileDirPath;
    }

    /**
     * Encoding scheme for the export files.
     * Defaults to XML.
     * @return the scheme
     */
    public static String getScheme() {
        return scheme;
    }

    /**
     * Encoding scheme for the export files.
     * Defaults to XML.
     * @param scheme the scheme to set
     */
    public static void setScheme(String scheme) {
        XMLExportContext.scheme = scheme;
    }

    /**
     * Version level of the scheme definition for the export files.
     * Defaults to latest released version.
     * @return the version
     */
    public static String getVersion() {
        return version;
    }

    /**
     * Version level of the scheme definition for the export files.
     * Defaults to latest released version.
     * @param version the version to set
     */
    public static void setVersion(String version) {
        XMLExportContext.version = version;
    }

    /**
     * @return the level
     */
    public static String getLevel() {
        return level;
    }

    /**
     * @param aLevel the level to set
     */
    public static void setLevel(String aLevel) {
        level = aLevel;
    }

    

    
}
