/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.List;

public abstract class BaseRDFExporter {
    protected PrintWriter frbrpw;
    protected PrintWriter efrbrpw;
    protected List<Integer> ids;
    protected Integer id;
    protected String fileName;
    private final String FILE_ROOT = "/usr/local/vfrbr/rdf/";
    public final String FRBER_COMMENT = "http://iflastandards.info/ns/fr/frbr/frbrer/";
    public final String RDF_DT_STRING = "rdf:datatype=\"&xsd;string\"";
    public final String RDF_DT_INTEGER = "rdf:datatype=\"&xsd;integer\"";
    private final String FRBR_POSTFIX = "-rdf-frbr.xml";
    private final String EFRBR_POSTFIX = "-rdf-efrbr.xml";
    private final Integer ENTITIES_PER_FILE = 3000;

    private final String FRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n" +
        "<rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n" +
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
        "    >\n";

    public BaseRDFExporter(List<Integer> ids, String fileName) {
        this.ids = ids;
        this.fileName = fileName;
    }

    private void flushWriters() {
        frbrpw.flush();
        efrbrpw.flush();
    }

    private void writeString(String value) {
        frbrpw.print(value);
        efrbrpw.print(value);
    }

    private void writeTriple(RDFExportTriple triple) {
        if(triple.getExportGroup().equals(RDFExportTriple.ExportGroup.FRBR)) {
            frbrpw.print(triple.exportToElement());
            efrbrpw.print(triple.exportToElement());
        } else if(triple.getExportGroup().equals(RDFExportTriple.ExportGroup.EFRBR)) {
            efrbrpw.print(triple.exportToElement());
        }
    }

    private void closeWriters() {
        frbrpw.close();
        efrbrpw.close();
    }

    public final void handleExport() {
        //add outer loop for each file
        int startIndex = 0;
        int endIndex = 0;
        int fileNumber = 0;
        while(startIndex < ids.size()) {
            fileNumber++;
            endIndex += ENTITIES_PER_FILE;
            if (endIndex > ids.size()) {
                endIndex = ids.size();
            }
            try {
                frbrpw = new PrintWriter(new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream(FILE_ROOT + fileName + fileNumber + FRBR_POSTFIX), "UTF-8")));
                efrbrpw = new PrintWriter(new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream(FILE_ROOT + fileName + fileNumber + EFRBR_POSTFIX), "UTF-8")));
                handleHeader();
                for(Integer tempId : ids.subList(startIndex, endIndex)) {
                    this.id = tempId;
                    handleBody();
                }
                handleFooter();
                this.flushWriters();
                this.closeWriters();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            startIndex = endIndex;
        }
    }

    protected final String getFRBRHeaderText() {
        return this.FRBR_HEADER_TEXT;
    }
    abstract protected String getEFRBRHeaderText();
    abstract protected String getDescriptionStart();

    protected final void handleHeader() {
        this.frbrpw.print(this.getFRBRHeaderText());
        this.efrbrpw.print(this.getEFRBRHeaderText());
        this.flushWriters();
    }

    abstract protected List<RDFExportTriple> getTriples();

    protected final void handleBody() {
        this.writeString(getDescriptionStart());
        List<RDFExportTriple> triples = getTriples();
        for(RDFExportTriple triple : triples) {
            this.writeTriple(triple);
        }
        this.flushWriters();
        this.writeString(getDescriptionEnd());
    }

    protected final String getDescriptionEnd() {
        return "    </rdf:Description>\n";
    }

    protected final void handleFooter() {
        this.writeString("</rdf:RDF>\n");
        this.flushWriters();
    }
}