/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleGeneral;
import edu.indiana.dlib.vfrbr.persist.dao.CorporateBodyDAO;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyDate;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyJpa;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyName;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyNote;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyNumber;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyPlace;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToManifestation;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToResponsibleParty;
import java.util.ArrayList;
import java.util.List;

public class CorporateBodyRDFExporter extends BaseRDFExporter{

    private DAOFactory daoFac;
    private CorporateBodyDAO corpDAO;

    private String EFRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n" +
        "<rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n" +
        "    xmlns:roles=\"http://RDVocab.info/roles\"\n" +
        "    xmlns:ElementsGr2=\"http://RDVocab.info/ElementsGr2\"\n" +
        "    xmlns:Elements=\"http://RDVocab.info/Elements\"\n" +
        "    xmlns:frad=\"http://iflastandards.info/ns/fr/frad/\"\n" +
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
        "    xmlns:vfrad=\"http://vfrad.info/ns/vfrad\"\n" +
        "    xmlns:vfrbr=\"http://vfrbr.info/ns/vfrbr\"\n" +
        "    >\n";

    public CorporateBodyRDFExporter(List<Integer> ids, String fileName, DAOFactory daoFac,
            CorporateBodyDAO corpDAO) {
        super(ids, fileName);
        this.daoFac = daoFac;
        this.corpDAO = corpDAO;
    }

    @Override
    protected String getEFRBRHeaderText() {
        return this.EFRBR_HEADER_TEXT;
    }

    @Override
    protected String getDescriptionStart() {
        return "    <rdf:Description rdf:about=\"http://vfrbr.info/corporatebody/" + id + "\">\n";
    }

    @Override
    protected List<RDFExportTriple> getTriples() {
        List<RDFExportTriple> triples = new ArrayList<RDFExportTriple>();
        CorporateBodyJpa corp = corpDAO.getById(id);

        List<String> workRefs = new ArrayList<String>();
        List<String> composers = new ArrayList<String>();
        List<String> librettists =  new ArrayList<String>();
        List<String> lyricist = new ArrayList<String>();
        for(WorkToResponsibleParty w2r : corp.getIsComposerOfWorks()) {
            String relUrl = "http://vfrbr.info/work/"+w2r.getSourceWork().getId();
            workRefs.add(relUrl);
            if(w2r.getRelRole() == null) {
                //do nothing
            } else if(w2r.getRelRole().equals("composer")) {
                composers.add(relUrl);
            } else if(w2r.getRelRole().equals("lyricist")) {
                lyricist.add(relUrl);
            } else if(w2r.getRelRole().equals("librettist")) {
                librettists.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2008", workRefs, null, RDFExportTripleGeneral.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2008 = is creator (corporate body) of"));

        List<String> exprRefs = new ArrayList<String>();
        List<String> arrangers = new ArrayList<String>();
        List<String> conductors = new ArrayList<String>();
        List<String> choreographers = new ArrayList<String>();
        List<String> musicians = new ArrayList<String>();
        List<String> singers = new ArrayList<String>();
        List<String> instrumentalists = new ArrayList<String>();
        List<String> translators = new ArrayList<String>();
        for(ExpressionToResponsibleParty e2r : corp.getIsRealizerOfExpressions()) {
            String relUrl = "http://vfrbr.info/expression/" + e2r.getSourceExpression().getId();
            exprRefs.add(relUrl);
            if(e2r.getRelRole().equals("arranger")) {
                arrangers.add(relUrl);
            } else if(e2r.getRelRole().equals("conductor")) {
                conductors.add(relUrl);
            } else if(e2r.getRelRole().equals("choreographer")) {
                choreographers.add(relUrl);
            } else if(e2r.getRelRole().equals("performer")) {
                musicians.add(relUrl);
            } else if(e2r.getRelRole().equals("vocalist")) {
                singers.add(relUrl);
            } else if(e2r.getRelRole().equals("instrumentalist")) {
                instrumentalists.add(relUrl);
            } else if(e2r.getRelRole().equals("translator")) {
                translators.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2012", exprRefs, null, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2012 = is realizer (corporate body) of"));

        List<String> manifRefs = new ArrayList<String>();
        List<String> editors = new ArrayList<String>();
        List<String> illustrators = new ArrayList<String>();
        List<String> lithographers = new ArrayList<String>();
        List<String> recordingEngineers = new ArrayList<String>();
        for(ManifestationToResponsibleParty m2r : corp.getIsProducerOfManifestations()) {
            String relUrl = "http://vfrbr.info/manifestation/" + m2r.getSourceManifestation().getId();
            manifRefs.add(relUrl);
            if(m2r.getRelRole() == null) {
                //do nothing
            } else if(m2r.getRelRole().equals("editor")) {
                editors.add(relUrl);
            } else if(m2r.getRelRole().equals("illustrator")) {
                illustrators.add(relUrl);
            } else if(m2r.getRelRole().equals("lithographer")) {
                lithographers.add(relUrl);
            } else if(m2r.getRelRole().equals("recordingEngineer")) {
                recordingEngineers.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2016", manifRefs, null, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2016 = is producer (corporate body) of"));

        List<String> name = new ArrayList<String>();
        name.add(corp.getAuthorizedName());
        triples.add(new RDFExportTripleGeneral("frbrer:P3043", name, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT +"P3043 = has name of corporate body"));

        List<String> numbers = new ArrayList<String>();
        for(CorporateBodyNumber cbn : corp.getNumbers()) {
            numbers.add(cbn.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P4044", numbers, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT +"P4044 = has number associated with the corporate body"));

        List<String> places = new ArrayList<String>();
        for(CorporateBodyPlace cbp : corp.getPlaces()) {
            places.add(cbp.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P4045", places, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT +"P4045 = has place associated with the corporate body"));

        List<String> dates = new ArrayList<String>();
        for(CorporateBodyDate cbd : corp.getDates()) {
            dates.add(cbd.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3046", dates, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, "P3046 = has date associated with the corporate body"));

        //EFRBR level
        triples.add(new RDFExportTripleGeneral("ElementsGr2:nameOfTheCorporateBody", name, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> variantNames = new ArrayList<String>();
        for(CorporateBodyName cbn : corp.getNames()) {
            if(cbn.getType().equals("variant")) {
                variantNames.add(cbn.getText());
            }
        }
        triples.add(new RDFExportTripleGeneral("ElementsGr2:variantNameForTheCorporateBody", variantNames, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("ElementsGr2:preferredNameForTheCorporateBody", name, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("ElementsGr2:placeAssociatedWithTheCorporateBody", places, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:numberAssociatedWithTheCorporateBody", numbers, RDF_DT_INTEGER, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> notes = new ArrayList<String>();
        for(CorporateBodyNote cbn : corp.getNotes()) {
            notes.add(cbn.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:note", notes, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("roles:composer", composers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:librettist", librettists, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:lyricist", lyricist, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:editor", editors, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:illustrator", illustrators, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:lithographer", lithographers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:recordingEngineer", recordingEngineers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:arrangerOfMusic", arrangers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:conductor", conductors, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:choreographer", choreographers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("vfrbr:musician", musicians, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:singer", singers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:instrumentalist", instrumentalists, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:translator", translators, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        this.daoFac.flushClearEntityManager();
        return triples;
    }
}
