/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleGeneral;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleMediumPerformance;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleRelationList;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.ValueWithQuantity;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.ExpressionDAO;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionDate;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionForm;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionJpa;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionKey;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionNote;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionPerformanceMedium;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionPerformancePlace;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionScoreType;
import edu.indiana.dlib.vfrbr.persist.entity.expression.ExpressionTitle;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.CorporateBodyJpa;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonJpa;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToManifestation;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToExpression;
import java.util.ArrayList;
import java.util.List;

public class ExpressionRDFExporter extends BaseRDFExporter{

    private DAOFactory daoFac;
    private ExpressionDAO exprDAO;
    private final String EFRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n" +
        "<rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n" +
        "    xmlns:roles=\"http://RDVocab.info/roles\"\n" +
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
        "    xmlns:Elements=\"http://RDVocab.info/Elements\"\n" +
        "    xmlns:vfrbr=\"http://vfrbr.info/ns/vfrbr\"\n" +
        "    xmlns:mo=\"http://purl.org/ontology/mo\"\n" +
        "    >\n";

    public ExpressionRDFExporter(List<Integer> ids, String fileName, DAOFactory daoFac,
            ExpressionDAO exprDAO) {
        super(ids, fileName);
        this.daoFac = daoFac;
        this.exprDAO = exprDAO;
    }

    @Override
    protected String getEFRBRHeaderText() {
        return this.EFRBR_HEADER_TEXT;
    }

    @Override
    protected String getDescriptionStart() {
        return "    <rdf:Description rdf:about=\"http://vfrbr.info/expression/" + id + "\">\n";
    }

    @Override
    protected List<RDFExportTriple> getTriples() {
        List<RDFExportTriple> triples = new ArrayList<RDFExportTriple>();
        ExpressionJpa expr = this.exprDAO.getById(id);

        List<String> workRefs = new ArrayList<String>();
        for(WorkToExpression w2e : expr.getIsRealization()) {
            workRefs.add("http://vfrbr.info/work/"+w2e.getSourceWork().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2002", workRefs, null, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2002 = is realization of"));

        List<String> titles = new ArrayList<String>();
        for(ExpressionTitle title:expr.getTitles()) {
            titles.add(title.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3008", titles, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT + "P3008 = has title expression"));

        List<String> forms = new ArrayList<String>();
        for(ExpressionForm form : expr.getForms()) {
            forms.add(form.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3009", forms, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3009 = form of expression"));

        List<String> dates = new ArrayList<String>();
        for(ExpressionDate date : expr.getDates()) {
            dates.add(date.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3010", dates, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT + "P3010 = has date of expression"));

        List<String> typeOfScore = new ArrayList<String>();
        for(ExpressionScoreType scoreType : expr.getScoreType()) {
            typeOfScore.add(scoreType.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3067", typeOfScore, RDF_DT_STRING,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE,
                FRBER_COMMENT + "P3067 = has type of score (musical notation)"));

        List<String> performanceMedium = new ArrayList<String>();
        List<ValueWithQuantity> performanceMediumWithQuantity = new ArrayList<ValueWithQuantity>();
        for(ExpressionPerformanceMedium epm : expr.getPerformanceMediums()) {
            performanceMedium.add(epm.getText());
            performanceMediumWithQuantity.add(new ValueWithQuantity(epm.getText(), epm.getQuantity()));
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3068", performanceMedium,
                RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.VALUE,
                FRBER_COMMENT + "P3068 = has medium of performance"));

        List<String> manifRefs = new ArrayList<String>();
        for(ExpressionToManifestation e2m : expr.getHasEmbodiments()) {
            manifRefs.add("http://vfrbr.info/manifestation/" + e2m.getTargetManifestation().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2003", manifRefs, null,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE,
                FRBER_COMMENT + "P2003 = is embodied in"));

        List<String> personRefs = new ArrayList<String>();
        List<String> corpRefs = new ArrayList<String>();
        List<String> arrangers = new ArrayList<String>();
        List<String> conductors = new ArrayList<String>();
        List<String> choreographers = new ArrayList<String>();
        List<String> musicians = new ArrayList<String>();
        List<String> performers = new ArrayList<String>();
        List<String> singers = new ArrayList<String>();
        List<String> instrumentalists = new ArrayList<String>();
        List<String> translators = new ArrayList<String>();
        for(ExpressionToResponsibleParty e2r : expr.getHasRealizers()) {
            String baseURL = "";
            Integer id = e2r.getTargetResponsibleParty().getId();
            if(e2r.getTargetResponsibleParty() instanceof PersonJpa) {
                personRefs.add("http://vfrbr.info/person/" + e2r.getTargetResponsibleParty().getId());
                baseURL = "http://vfrbr.info/person/";
            }
            if(e2r.getTargetResponsibleParty() instanceof CorporateBodyJpa) {
                corpRefs.add("http://vfrbr.info/corporatebody/" + e2r.getTargetResponsibleParty().getId());
                baseURL = "http://vfrbr.info/corporatebody/";
            }
            String repURL = baseURL + id;
            if(e2r.getRelRole().equals("arranger")) {
                arrangers.add(repURL);
            } else if(e2r.getRelRole().equals("performer")) {
                performers.add(repURL);
            } else if(e2r.getRelRole().equals("conductor")) {
                conductors.add(repURL);
            } else if(e2r.getRelRole().equals("vocalist")) {
                singers.add(repURL);
            } else if(e2r.getRelRole().equals("instrumentalist")) {
                instrumentalists.add(repURL);
            } else if(e2r.getRelRole().equals("translator")) {
                translators.add(repURL);
            } else if(e2r.getRelRole().equals("choreographer")) {
                choreographers.add(repURL);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2011", corpRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2011 = is realized by (corporate body)"));
        triples.add(new RDFExportTripleGeneral("frbrer:P2013", personRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2013 = is realized by (person)"));


        //EFRBR level
        List<String> notes = new ArrayList<String>();
        for(ExpressionNote en : expr.getNotes()) {
            notes.add(en.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:note", notes, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> placeOfPerformance = new ArrayList<String>();
        for(ExpressionPerformancePlace epp : expr.getPerformancePlace()) {
            placeOfPerformance.add(epp.getText());
        }
        triples.add(new RDFExportTripleGeneral("vfrbr:placeOfPerformance", placeOfPerformance, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        ArrayList<String> key = new ArrayList<String>();
        for(ExpressionKey ek : expr.getKeys()) {
            key.add(ek.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:key", key, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:title", titles, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:formOfExpression", forms, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:dateOfExpression", dates, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:formatOfNotatedMusicExpression", typeOfScore, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleMediumPerformance(performanceMediumWithQuantity));

        triples.add(new RDFExportTripleGeneral("vfrbr:placeOfPerformance", placeOfPerformance, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:genreFormStyle", forms, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("roles:arrangerOfMusicExpression", arrangers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleRelationList("roles:conductorExpression", conductors));
        triples.add(new RDFExportTripleRelationList("roles:choreographerExpression", choreographers));
        triples.add(new RDFExportTripleRelationList("vfrbr:musicianExpression", musicians));
        triples.add(new RDFExportTripleRelationList("roles:performerExpression", performers));
        triples.add(new RDFExportTripleRelationList("roles:singerExpression", singers));
        triples.add(new RDFExportTripleRelationList("roles:instrumentalistExpression", instrumentalists));
        triples.add(new RDFExportTripleRelationList("roles:translatorExpression", translators));

        daoFac.flushClearEntityManager();
        return triples;
    }

}
