/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleGeneral;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleRelationList;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.ManifestationDAO;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationAccessAddress;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationAccessMode;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationAccompanyingLanguage;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationCaptureMode;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationCarrierDimension;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationCarrierForm;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationIdentifier;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationJpa;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationNote;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationPhysicalMedium;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationPlayingSpeed;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationPublicationJpa;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationPublicationPublisher;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationReproductionCharacteristic;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationSeriesStatement;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationSoundKind;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.ManifestationTapeConfiguration;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonJpa;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.ResponsiblePartyJpa;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToManifestation;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToResponsibleParty;
import java.util.ArrayList;
import java.util.List;

public class ManifestationRDFExporter extends BaseRDFExporter {

    private DAOFactory daoFac;
    private ManifestationDAO manifDAO;

    private final String EFRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n" +
        "<rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n" +
        "    xmlns:Elements=\"http://RDVocab.info/Elements\"\n" +
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
        "    xmlns:vfrbr=\"http://vfrbr.info/ns/vfrbr\"\n" +
        "    xmlns:roles=\"http://RDVocab.info/roles\"\n" +
        "    xmlns:mo=\"http://purl.org/ontology/mo/\"\n" +
        "    >\n";


    public ManifestationRDFExporter(List<Integer> ids, String fileName, DAOFactory daoFac,
            ManifestationDAO manifDAO) {
        super(ids, fileName);
        this.daoFac = daoFac;
        this.manifDAO = manifDAO;
    }
    

    @Override
    protected String getEFRBRHeaderText() {
        return EFRBR_HEADER_TEXT;
    }

    @Override
    protected String getDescriptionStart() {
        return "    <rdf:Description rdf:about=\"http://vfrbr.info/manifestation/" + id + "\">\n";
    }

    @Override
    protected List<RDFExportTriple> getTriples() {
        List<RDFExportTriple> triples = new ArrayList<RDFExportTriple>();
        ManifestationJpa manif = this.manifDAO.getById(id);

        List<String> exprRefs = new ArrayList<String>();
        for(ExpressionToManifestation e2m : manif.getIsEmbodiment()) {
            exprRefs.add("http://vfrbr.info/expression/" + e2m.getSourceExpression().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2004", exprRefs, null,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE,
                FRBER_COMMENT+"P2004 = is embodiment of"));

        List<String> manifTitle = new ArrayList<String>();
        manifTitle.add(manif.getTitles().get(0).getText());
        triples.add(new RDFExportTripleGeneral("frbrer:P3020", manifTitle, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3020 = has title of the manifestation"));

        List<String> placeOfPub = new ArrayList<String>();
        List<String> pub = new ArrayList<String>();
        List<String> pubDate = new ArrayList<String>();
        if(manif.getPublications().size() > 0) {
            ManifestationPublicationJpa mpub = manif.getPublications().get(0);
            if(mpub.getPlace() != null) {
                placeOfPub.add(mpub.getPlace().getText());
            }
            if(mpub.getPublisher() != null) {
                pub.add(mpub.getPublisher().getText());
            }
            if(mpub.getDate() != null) {
                pubDate.add(mpub.getDate().getText());
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3057", placeOfPub, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3057 = has place of publication or distibutor"));
        triples.add(new RDFExportTripleGeneral("frbrer:P3056", pub, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3056 = has publisher or distributor"));
        triples.add(new RDFExportTripleGeneral("frbrer:P3055", pubDate, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3055 = has date of publication or distribution"));

        List<String> seriesStmt = new ArrayList<String>();
        for(ManifestationSeriesStatement mss : manif.getSeriesStmts()) {
            seriesStmt.add(mss.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3022", seriesStmt, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3022 = has series statement"));

        List<String> formOfCarrier = new ArrayList<String>();
        for(ManifestationCarrierForm mcf : manif.getCarrierForms()) {
            formOfCarrier.add(mcf.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3032", formOfCarrier, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3023 = has form of carrier"));

        List<String> physicalMedium = new ArrayList<String>();
        for(ManifestationPhysicalMedium mpm : manif.getPhysicalMediums()) {
            physicalMedium.add(mpm.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3025", physicalMedium, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3025 = has physical medium"));

        List<String> captureMode = new ArrayList<String>();
        for(ManifestationCaptureMode mcm : manif.getCaptureModes()) {
            captureMode.add(mcm.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3026", captureMode, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3026 = has capture mode"));

        List<String> dimensionOfCarrier = new ArrayList<String>();
        for(ManifestationCarrierDimension mcd : manif.getCarrierDimensions()) {
            dimensionOfCarrier.add(mcd.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3027", dimensionOfCarrier, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3027 = has dimensions of the carrier"));

        List<String> manifIdent = new ArrayList<String>();
        for(ManifestationIdentifier mi : manif.getIdentifiers()) {
            manifIdent.add(mi.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3028", manifIdent, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3028 = has manifestation identifier"));

        List<String> playingSpeed = new ArrayList<String>();
        for(ManifestationPlayingSpeed mps : manif.getPlayingSpeeds()) {
            playingSpeed.add(mps.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3080", playingSpeed, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3080 = has playing speed (sound recording"));

        List<String> tapeConfig = new ArrayList<String>();
        for(ManifestationTapeConfiguration mtc : manif.getTapeConfigs()) {
            tapeConfig.add(mtc.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3083", tapeConfig, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3083 = has tape configuration (sound recording)"));

        List<String> kindOfSound = new ArrayList<String>();
        for(ManifestationSoundKind msk : manif.getSoundKinds()) {
            kindOfSound.add(msk.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3084", kindOfSound, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3084 = has kind of sound (sound recording)"));

        List<String> reproductionCharacteristic = new ArrayList<String>();
        for(ManifestationReproductionCharacteristic mrc : manif.getReproChars()) {
            reproductionCharacteristic.add(mrc.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3085", reproductionCharacteristic, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3085 = has special reproduction characteristic (sound recording)"));

        List<String> modeOfAccess = new ArrayList<String>();
        for(ManifestationAccessMode mam : manif.getAccessModes()) {
            modeOfAccess.add(mam.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3095", modeOfAccess, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT + "P3095 = has mode of access (remote electronic resource)"));

        List<String> accessAddress = new ArrayList<String>();
        for(ManifestationAccessAddress maa : manif.getAccessAddresses()) {
            accessAddress.add(maa.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3096", accessAddress, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P3096 = has access address (remote electronic resource)"));

        List<String> corpRefs = new ArrayList<String>();
        for(ManifestationToResponsibleParty m2r : manif.getHasProducerCorporateBodies()) {
            corpRefs.add("http://vfrbr.info/corporatebody/" + m2r.getTargetResponsibleParty().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2015", corpRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2015 = is producted by (corporate body)"));

        List<String> personRefs = new ArrayList<String>();
        for(ManifestationToResponsibleParty m2r : manif.getHasProducerPersons()) {
            personRefs.add("http://vfrbr.info/person/" + m2r.getTargetResponsibleParty().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2017", personRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2017 = is produced by (person)"));

        //EFRBR level
        List<String> language = new ArrayList<String>();
        for(ManifestationAccompanyingLanguage mal : manif.getAccompLangs()) {
            language.add("http://id.loc.gov/vocabulary/iso639-2/" + mal.getNormal() + ".rdf");
        }
        triples.add(new RDFExportTripleGeneral("vfrbr:languageOfAccompanyingMaterials", language, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        triples.add(new RDFExportTripleGeneral("Elements:titleManifestation", manifTitle, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:placeOfPublicationManifestation", placeOfPub, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:publishersNameManifestation", pub, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:dateOfPublicationManifestation", pubDate, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:seriesStatementManifestation", seriesStmt, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:physicalMedium", physicalMedium, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:carrierTypeManifestation", formOfCarrier, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:recordingMediumManifestation", captureMode, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:dimensionsManifestation", dimensionOfCarrier, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:identifierForTheManifestation", manifIdent, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:playingSpeedManifestation", playingSpeed, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:tapeConfigurationManifestation", tapeConfig, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:modeOfAccess", modeOfAccess, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> urls = new ArrayList<String>();
        urls.add("http://vfrbr.info/manifestation/" + manif.getId());
        triples.add(new RDFExportTripleGeneral("Elements:uniformResourceLocatorManifestation", urls, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        List<String> notes = new ArrayList<String>();
        for(ManifestationNote mn : manif.getNotes()) {
            notes.add(mn.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:noteManifestation", notes, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> editors = new ArrayList<String>();
        List<String> illustrators = new ArrayList<String>();
        List<String> recordingEngineers = new ArrayList<String>();

        List<ManifestationToResponsibleParty> m2rs = new ArrayList<ManifestationToResponsibleParty>();
        m2rs.addAll(manif.getHasProducerPersons());
        m2rs.addAll(manif.getHasProducerCorporateBodies());
        for(ManifestationToResponsibleParty m2r : m2rs) {
            String relRole = m2r.getRelRole();
            String respType = this.getResponsiblePartyType(m2r.getTargetResponsibleParty());
            if(relRole == null) {
                continue;
            } else if(relRole.equals("editor")) {
                editors.add("http://vfrbr.info/" + respType + "/" + m2r.getTargetResponsibleParty().getId());
            } else if(relRole.equals("illustrator")) {
                illustrators.add("http://vfrbr.info/" + respType + "/" + m2r.getTargetResponsibleParty().getId());
            } else if(relRole.equals("recordingEngineer")) {
                recordingEngineers.add("http://vfrbr.info/" + respType + "/" + m2r.getTargetResponsibleParty().getId());
            }
        }

        triples.add(new RDFExportTripleRelationList("roles:editor", editors));

        triples.add(new RDFExportTripleGeneral("roles:illustrator", illustrators, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        triples.add(new RDFExportTripleGeneral("roles:recordingEngineer", recordingEngineers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        this.daoFac.flushClearEntityManager();
        return triples;
    }

    private String getResponsiblePartyType(ResponsiblePartyJpa rp) {
        String fullClassName = rp.getClass().getName().toLowerCase();
        return fullClassName.substring(fullClassName.lastIndexOf('.') + 1, fullClassName.length() - 3);
    }
}
