/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleGeneral;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleValueList;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.PersonDAO;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonDate;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonDesignation;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonJpa;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonName;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonNote;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.PersonTitle;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToResponsibleParty;
import java.util.ArrayList;
import java.util.List;

public class PersonRDFExporter extends BaseRDFExporter {

    private PersonDAO personDAO;
    private DAOFactory daoFac;


    private final String EFRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "   <!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n" +
        "   <rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n" +
        "       xmlns:ElementsGr2=\"http://RDVocab.info/ElementsGr2\"\n" +
        "       xmlns:Elements=\"http://RDVocab.info/Elements\"\n" +
        "       xmlns:roles=\"http://RDVocab.info/roles\"\n" +
        "       xmlns:frad=\"http://iflastandards.info/ns/fr/frad/\"\n" +
        "       xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "       xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n" +
        "       xmlns:vfrad=\"http://vfrad.info/ns/vfrad\"\n" +
        "       xmlns:vfrbr=\"http://vfrbr.info/ns/vfrbr\"\n" +
        "   >\n";

    public PersonRDFExporter(List<Integer> ids, String fileName, DAOFactory daoFac,
            PersonDAO personDAO) {
        super(ids, fileName);
        this.daoFac = daoFac;
        this.personDAO = personDAO;
    }
    
    @Override
    protected List<RDFExportTriple> getTriples() {
        List<RDFExportTriple> triples = new ArrayList<RDFExportTriple>();
        PersonJpa person = this.personDAO.getById(id);

        List<String> authorizedName = new ArrayList<String>();
        authorizedName.add(person.getAuthorizedName());
        triples.add(new RDFExportTripleGeneral("frbrer:P3039", authorizedName, null,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE,
                FRBER_COMMENT + "P3039 = has name of person"));

        List<String> dates = new ArrayList<String>();
        for(PersonDate pd : person.getDates()) {
            dates.add(pd.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3040", dates, RDF_DT_STRING,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE,
                FRBER_COMMENT + "P3040 = has dates of person"));

        List<String> titles = new ArrayList<String>();
        for(PersonTitle pt : person.getTitles()) {
            titles.add(pt.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3041", titles, RDF_DT_STRING,
                RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE,
                FRBER_COMMENT + "P3041 = has title of person"));

        List<String> otherDesignations = new ArrayList<String>();
        for(PersonDesignation pd : person.getDesignations()) {
            otherDesignations.add(pd.getText());
        }

        List<String> workRefs = new ArrayList<String>();
        List<String> composers = new ArrayList<String>();
        List<String> librettists =  new ArrayList<String>();
        List<String> lyricist = new ArrayList<String>();
        for(WorkToResponsibleParty w2r : person.getIsComposerOfWorks()) {
            String relUrl = "http://vfrbr.info/work/" + w2r.getSourceWork().getId();
            workRefs.add(relUrl);
            if(w2r.getRelRole() == null) {
                //do nothing
            } else if(w2r.getRelRole().equals("composer")) {
                composers.add(relUrl);
            } else if(w2r.getRelRole().equals("lyricist")) {
                lyricist.add(relUrl);
            } else if(w2r.getRelRole().equals("librettist")) {
                librettists.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2010", workRefs, null, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2010 = is creator (person) of"));


        List<String> manifRefs = new ArrayList<String>();
        List<String> editors = new ArrayList<String>();
        List<String> illustrators = new ArrayList<String>();
        List<String> lithographers = new ArrayList<String>();
        List<String> recordingEngineers = new ArrayList<String>();
        for(ManifestationToResponsibleParty m2r : person.getIsProducerOfManifestations()) {
            String relUrl = "http://vfrbr.info/manifestation/" + m2r.getSourceManifestation().getId();
            manifRefs.add(relUrl);
            if(m2r.getRelRole() == null) {
                //do nothing
            } else if(m2r.getRelRole().equals("editor")) {
                editors.add(relUrl);
            } else if(m2r.getRelRole().equals("illustrator")) {
                illustrators.add(relUrl);
            } else if(m2r.getRelRole().equals("lithographer")) {
                lithographers.add(relUrl);
            } else if(m2r.getRelRole().equals("recordingEngineer")) {
                recordingEngineers.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2018", manifRefs, null, RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2018 = is producer (person) of"));

        List<String> exprRefs = new ArrayList<String>();
        List<String> arrangers = new ArrayList<String>();
        List<String> conductors = new ArrayList<String>();
        List<String> choreographers = new ArrayList<String>();
        List<String> musicians = new ArrayList<String>();
        List<String> singers = new ArrayList<String>();
        List<String> instrumentalists = new ArrayList<String>();
        List<String> translators = new ArrayList<String>();
        for(ExpressionToResponsibleParty e2r : person.getIsRealizerOfExpressions()) {
            String relUrl = "http://vfrbr.info/expression/"+e2r.getSourceExpression().getId();
            exprRefs.add(relUrl);
            if(e2r.getRelRole().equals("arranger")) {
                arrangers.add(relUrl);
            } else if(e2r.getRelRole().equals("conductor")) {
                conductors.add(relUrl);
            } else if(e2r.getRelRole().equals("choreographer")) {
                choreographers.add(relUrl);
            } else if(e2r.getRelRole().equals("performer")) {
                musicians.add(relUrl);
            } else if(e2r.getRelRole().equals("vocalist")) {
                singers.add(relUrl);
            } else if(e2r.getRelRole().equals("instrumentalist")) {
                instrumentalists.add(relUrl);
            } else if(e2r.getRelRole().equals("translator")) {
                translators.add(relUrl);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2014", exprRefs, "", RDFExportTriple.ExportGroup.FRBR,
                RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P2014 = is realizer (person) of"));


        //EFRBR level
        triples.add(new RDFExportTripleGeneral("roles:composer", composers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:librettist", librettists, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:lyricist", lyricist, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:editor", editors, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:illustrator", illustrators, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:lithographer", lithographers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:recordingEngineer", recordingEngineers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:arrangerOfMusic", arrangers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:conductor", conductors, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:choreographer", choreographers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("vfrbr:musician", musicians, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:singer", singers, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:instrumentalist", instrumentalists, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));
        triples.add(new RDFExportTripleGeneral("roles:translator", translators, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        triples.add(new RDFExportTripleGeneral("ElementsGr2:nameOfThePerson", authorizedName, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> variantNames = new ArrayList<String>();
        for(PersonName pn : person.getNames()) {
            if(pn.getType().equals("variant")) {
                variantNames.add(pn.getText());
            }
        }
        triples.add(new RDFExportTripleGeneral("ElementsGr2:variantNameForThePerson", variantNames, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleValueList("ElementsGr2:titleOfPerson", titles));

        triples.add(new RDFExportTripleValueList("ElementsGr2:otherDesignationAssociatedWithThePerson", otherDesignations));

        List<String> notes = new ArrayList<String>();
        for(PersonNote pn : person.getNotes()) {
            notes.add(pn.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:note", notes, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        this.daoFac.flushClearEntityManager();
        return triples;
    }

    @Override
    protected String getEFRBRHeaderText() {
        return this.EFRBR_HEADER_TEXT;
    }

    @Override
    protected String getDescriptionStart() {
        return "    <rdf:Description rdf:about=\"http://vfrbr.info/person/" + this.id +"\">\n";
    }
}
