/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.entityExporter;

import edu.indiana.dlib.vfrbr.export.lookup.LocalFileSubjectLookup;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTriple;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleGeneral;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleMediumPerformance;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.RDFExportTripleRelationList;
import edu.indiana.dlib.vfrbr.export.rdf.tripleWriter.ValueWithQuantity;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.WorkDAO;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkDate;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkDesignation;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkForm;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkJpa;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkKey;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkNote;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkPerformanceMedium;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkSubjectHeading;
import edu.indiana.dlib.vfrbr.persist.entity.work.WorkTitle;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToExpression;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToResponsibleParty;
import java.util.ArrayList;
import java.util.List;

public class WorkRDFExporter extends BaseRDFExporter {

    private DAOFactory daoFac;
    private WorkDAO workDAO;
    private final String EFRBR_HEADER_TEXT =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"+
        "<!DOCTYPE rdf:RDF [<!ENTITY xsd \"http://www.w3.org/2001/XMLSchema#\">]>\n"+
        "<rdf:RDF xmlns:frbrer=\"http://iflastandards.info/ns/fr/frbr/frbrer/\"\n"+
        "    xmlns:roles=\"http://RDVocab.info/roles\"\n" +
        "    xmlns:Elements=\"http://RDVocab.info/Elements\"\n"+
        "    xmlns:frad=\"http://iflastandards.info/ns/fr/frad/\"\n"+
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"+
        "    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"\n"+
        "    xmlns:mo=\"http://purl.org/ontology/mo\"\n" +
        "    xmlns:vfrbr=\"http://vfrbr.info/ns/vfrbr\"\n"+
        "    xmlns:vfrad=\"http://vfrad.info/ns/vfrad\"\n"+
        "    >\n";

    public WorkRDFExporter(List<Integer> ids, String fileName, DAOFactory daoFac,
            WorkDAO workDAO) {
        super(ids, fileName);
        this.daoFac = daoFac;
        this.workDAO = workDAO;
    }

    @Override
    protected String getEFRBRHeaderText() {
        return this.EFRBR_HEADER_TEXT;
    }

    @Override
    protected String getDescriptionStart() {
        return "    <rdf:Description rdf:about=\"http://vfrbr.info/work/" + id + "\">\n";
    }

    @Override
    protected List<RDFExportTriple> getTriples() {
        List<RDFExportTriple> triples = new ArrayList<RDFExportTriple>();
        WorkJpa work = this.workDAO.getById(id);

        List<String> titleAuth = new ArrayList<String>();
        List<String> types = new ArrayList<String>();
        types.add("uniform"); types.add("titleproper"); types.add(null);
        titleAuth.add(work.getTitlesOrderedByType(types).get(0).getText());
        triples.add(new RDFExportTripleGeneral("frbrer:P3001", titleAuth, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3001 = has title of the work"));

        List<String> form = new ArrayList<String>();
        for(WorkForm wf : work.getForms()) {
            form.add(wf.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3002", form, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3002 = has form of work"));

        List<String> date = new ArrayList<String>();
        for(WorkDate wd : work.getDates()) {
            date.add(wd.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3003", date, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3003 = has date of the work"));

        List<String> performanceMedium = new ArrayList<String>();
        List<ValueWithQuantity> performanceMediumWithQuantity = new ArrayList<ValueWithQuantity>();
        for(WorkPerformanceMedium wpm : work.getPerformanceMediums()) {
            performanceMedium.add(wpm.getText());
            performanceMediumWithQuantity.add(new ValueWithQuantity(wpm.getText(), wpm.getQuantity()));
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3059", performanceMedium, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3059 = has medium of performance"));

        List<String> designation = new ArrayList<String>();
        for(WorkDesignation wd : work.getDesignations()) {
            designation.add(wd.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3060", designation, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3060 = has numerical designation"));

        List<String> key = new ArrayList<String>();
        for(WorkKey wk : work.getKeys()) {
            key.add(wk.getText());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3061", key, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3061 = has key"));

        List<String> localSubjects = new ArrayList<String>();
        List<String> authSubjects = new ArrayList<String>();
        for(WorkSubjectHeading wsh : work.getSubjectHeadings()) {
            String subjectFromEntity = wsh.getText();
            String result = LocalFileSubjectLookup.lookup(subjectFromEntity);
            if(result == null) {
                localSubjects.add(subjectFromEntity);
            } else {
                authSubjects.add(result);
            }
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P3050", authSubjects, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT + "P3050 = has subject of the work"));
        triples.add(new RDFExportTripleGeneral("frbrer:P3050", localSubjects, RDF_DT_STRING, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.VALUE, FRBER_COMMENT+"P3050 = has subject of the work"));

        List<String> exprRefs = new ArrayList<String>();
        for(WorkToExpression w2e : work.getHasRealizations()) {
            exprRefs.add("http://vfrbr.info/expression/" + w2e.getTargetExpression().getId());
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2001", exprRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2001 = is realized through"));

        List<String> corpRefs = new ArrayList<String>();
        List<String> composers = new ArrayList<String>();
        List<String> librettists = new ArrayList<String>();
        List<String> lyricists = new ArrayList<String>();
        for(WorkToResponsibleParty w2r : work.getHasCreatorCorporations()) {
            String relUrl = "http://vfrbr.info/corporatebody/" + w2r.getTargetResponsibleParty().getId();
            corpRefs.add(relUrl);
            if(w2r.getRelRole().equals("librettist")) {
                librettists.add(relUrl);
            }
            if(w2r.getRelRole().equals("lyricist")) {
                lyricists.add(relUrl);
            }
        }
        for(WorkToResponsibleParty w2r : work.getHasComposerCorporations()) {
            String relUrl = "http://vfrbr.info/corporatebody/" + w2r.getTargetResponsibleParty().getId();
            corpRefs.add(relUrl);
            composers.add(relUrl);
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2007", corpRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2007 = is created by (corporate body)"));

        List<String> personRefs = new ArrayList<String>();
        for(WorkToResponsibleParty w2r : work.getHasCreatorPersons()) {
            String relUrl = "http://vfrbr.info/person/" + w2r.getTargetResponsibleParty().getId();
            personRefs.add(relUrl);
            if(w2r.getRelRole().equals("librettist")) {
                librettists.add(relUrl);
            }
            if(w2r.getRelRole().equals("lyricist")) {
                lyricists.add(relUrl);
            }
        }
        for(WorkToResponsibleParty w2r : work.getHasComposerPersons()) {
            String relUrl = "http://vfrbr.info/person/" + w2r.getTargetResponsibleParty().getId();
            personRefs.add(relUrl);
            composers.add(relUrl);
        }
        triples.add(new RDFExportTripleGeneral("frbrer:P2009", personRefs, null, RDFExportTriple.ExportGroup.FRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, FRBER_COMMENT+"P2009 = is created by (person)"));

        //efrbr level
        List<String> notes = new ArrayList<String>();
        for(WorkNote wn : work.getNotes()) {
            notes.add(wn.getText());
        }
        triples.add(new RDFExportTripleGeneral("Elements:note", notes, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:preferredTitleForTheWork", titleAuth, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        List<String> variantTitles = new ArrayList<String>();
        List<String> titleProper = new ArrayList<String>();
        for(WorkTitle wt : work.getTitles()) {
            if(wt.getType() == null) {
                //ignore case
            } else if(wt.getType().equals("variant")) {
                variantTitles.add(wt.getText());
            } else if(wt.getType().equals("titleproper")) {
                titleProper.add(wt.getText());
            }
        }
        triples.add(new RDFExportTripleGeneral("Elements:variantTitleForTheWork", variantTitles, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));
        triples.add(new RDFExportTripleGeneral("Elements:titleProper", titleProper, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:formOfWork", form, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:dateOfWork", date, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleMediumPerformance(performanceMediumWithQuantity));

        triples.add(new RDFExportTripleGeneral("Elements:numericDesignationOfAMusicWork", designation, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("Elements:key", key, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));

        triples.add(new RDFExportTripleGeneral("vfrbr:subjectOfTheWork", localSubjects, RDF_DT_STRING, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.VALUE, null));
        triples.add(new RDFExportTripleGeneral("vfrbr:subjectOfTheWork", authSubjects, null, RDFExportTriple.ExportGroup.EFRBR, RDFExportTripleGeneral.ElementForm.RESOURCE, null));

        triples.add(new RDFExportTripleRelationList("roles:composer", composers));

        triples.add(new RDFExportTripleRelationList("roles:librettist", librettists));

        triples.add(new RDFExportTripleRelationList("roles:lyricist", lyricists));

        this.daoFac.flushClearEntityManager();
        return triples;
    }
}
