/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.exporter;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.export.context.StructureEmbodiedContext;
import edu.indiana.dlib.vfrbr.export.xml.writer.EfrbrWriter;
import edu.indiana.dlib.vfrbr.export.xml.writer.StructureWriter;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.ExpressionToManifestationDAO;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import java.util.ArrayList;
import java.util.List;
import javax.xml.stream.XMLStreamException;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class StructEmbodiedExporter {

    private static final Logger log =
            Logger.getLogger(StructEmbodiedExporter.class);


    private final StructureEmbodiedContext structContext;

    public StructEmbodiedExporter(StructureEmbodiedContext embodiedInContext) {

        this.structContext = embodiedInContext;

    }

    public final void exportEmbodiedIn()
            throws FileNotFoundException,
                   UnsupportedEncodingException,
                   XMLStreamException {

        // no structContext, no export
        if (null == structContext) {

            log.error(
                    "*** exportEmbodiedIn "
                    + "called with null expr2ManifContext: aborting");

        } else {
            List<Integer> idList = new ArrayList<Integer>();

            DAOFactory daoFactory = new DAOFactory();

            ExpressionToManifestationDAO expr2ManifDAO =
                    daoFactory.newExpressionToManifestationDAO();

            /*
             * first get the idList of expr2Manif,
             * all or a sample
             */

            if (structContext.isSampleExport()) {
                if (log.isInfoEnabled()) {
                    log.info("  -- sample export of "
                            + "embodiedIn structure relations (expr2Manif): ["
                            + structContext.getSampleFromIndex()
                            + ", "
                            + structContext.getSampleToIndex()
                            + "]");
                }

                // only export a sample size of the entities
                idList = expr2ManifDAO.getTypeIds("embodiedIn").
                        subList(structContext.getSampleFromIndex(),
                                structContext.getSampleToIndex());

            } else {
                if (log.isInfoEnabled()) {
                    log.info("  -- export of all "
                            + "embodiedIn structure relations (expr2Manif)");
                }

                // export all the entities
                idList = expr2ManifDAO.getTypeIds("embodiedIn");

            }
            if (log.isInfoEnabled()) {
                log.info("     retrieved idList of size: "
                        + idList.size());
            }

            // done getting idList, close this EntityManager
            daoFactory.closeEntityManager();

            /*
             * then export the idList of expr2Manif
             * into single file
             * or multiple files of record count per file
             */

            if (structContext.getRecsPerFile() > 0) {
                // multiple files of recsPerFile

                if (log.isInfoEnabled()) {
                    log.info("    -- exporting muliple files of "
                            + structContext.getRecsPerFile()
                            + " records per file.");
                }

                // name the files
                // with appended two digit zero filled sequence number
                int fileSeqNum = 1;
                // from and to of the sublist range
                int fromIndex = 0;
                int toIndex = 0;

                while (fromIndex < idList.size()) {
                    // more list to process

                    // try for a full sized chunk
                    toIndex = fromIndex + structContext.getRecsPerFile();
                    // if we point off the end of the list
                    if (toIndex > idList.size()) {
                        // only go to the end of the list
                        toIndex = idList.size();
                    }

                    /*
                     * note: List.subList(fromIndex, toIndex)
                     * is fromIndex inclusive and toIndex exclusive
                     */

                    if (log.isInfoEnabled()) {
                        log.info("      -- exporting subList range ["
                                + fromIndex
                                + ", "
                                + toIndex
                                + "]");
                    }

                    // export this range
                    exportThese(daoFactory,
                                String.format(
                            structContext.getFileStem() + "%02d.xml",
                            fileSeqNum++),
                                idList.subList(fromIndex,
                                               toIndex));

                    // and start the next range from the excluded end
                    fromIndex = toIndex;
                } // of while more list to process

                if (log.isInfoEnabled()) {
                    log.info("    ---- done.\n");
                }
            } // of multiple files
            else {
                // one file of all recs

                if (log.isInfoEnabled()) {
                    log.info("    -- exporting one file of "
                            + idList.size()
                            + " embodiedIn structure relations (expr2Manif).");
                }

                exportThese(daoFactory,
                            structContext.getFileStem() + ".xml",
                            idList);

                if (log.isInfoEnabled()) {
                    log.info("    ---- done.\n");
                }
            }
        }
    }

    /**
     *  Export a set of expr2manifs.
     *
     * @param expr2manifDAO an intitialized expr2manifDAO.
     * @param fileName pathed fileName to create.
     * @param embodiedInIdList List of Integer embodiedIn id to export.
     * @throws FileNotFoundException if thrown.
     * @throws UnsupportedEncodingException if thrown.
     * @throws XMLStreamException if thrown.
     */
    private void exportThese(
            final DAOFactory daoFactory,
            final String fileName,
            final List<Integer> embodiedInIdList)
            throws FileNotFoundException,
                   UnsupportedEncodingException,
                   XMLStreamException {

        // export the idList to the fileName
        // shared OutputStream
        final OutputStream outputStream =
                new FileOutputStream(XMLExportContext.getFileDirPath() + fileName);

        // PrintWriter to OutputStream
        final PrintWriter xmlOut =
                new PrintWriter(new BufferedWriter(new OutputStreamWriter(
                outputStream, "UTF-8")));

        // XMLStreamWriter to OuputStream
//        final XMLStreamWriter xmlWrite =
//                XMLOutputFactory.newInstance().createXMLStreamWriter(
//                outputStream, "UTF-8");

        EfrbrWriter efrbrWriter = new EfrbrWriter();

        StructureWriter structWriter = new StructureWriter();

        // getting DAO also creates new EntityManager
        ExpressionToManifestationDAO expr2ManifDAO =
                daoFactory.newExpressionToManifestationDAO();

        if (log.isInfoEnabled()) {
            log.info("        exporting "
                    + embodiedInIdList.size()
                    + " expr2Manif to "
                    + XMLExportContext.getFileDirPath() + fileName);
        }

        // write the document start
        efrbrWriter.writeDocumentStart(xmlOut);

        // write the recordSet start
        efrbrWriter.writeRecordSetStart(xmlOut, null);

        // write the relations start
        efrbrWriter.writeRelationshipsStart(xmlOut);

        // write the structured start
        structWriter.writeEfrbrStructureStart(xmlOut);

        // write out the idList of ExpressionToManifestations
        for (Integer expr2manifIdInt : embodiedInIdList) {

            structWriter.writeEfrbrEmbodiedIn(xmlOut,
                                         expr2ManifDAO.getById(expr2manifIdInt));
        }

        // write the structured end
        structWriter.writeEfrbrStructureEnd(xmlOut);

        // write the entities end
        efrbrWriter.writeRelationshipsEnd(xmlOut);

        // write the recordSet end
        efrbrWriter.writeRecordSetEnd(xmlOut);

        // flush the stream
        xmlOut.flush();
        xmlOut.close();

        // de-reference the dao
        expr2ManifDAO = null;
        // and close this EntityManager
        daoFactory.closeEntityManager();
    }
}
