/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.exporter;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.export.context.WorkContext;
import edu.indiana.dlib.vfrbr.export.xml.writer.EfrbrWriter;
import edu.indiana.dlib.vfrbr.export.xml.writer.WorkWriter;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.WorkDAO;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import java.util.ArrayList;
import java.util.List;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class WorkExporter {

    private static final Logger log = Logger.getLogger(WorkExporter.class);

    private final WorkContext workContext;

    public WorkExporter(WorkContext workContext) {

        this.workContext = workContext;

    }

    public final void exportWorks()
            throws FileNotFoundException,
                   UnsupportedEncodingException,
                   XMLStreamException {

        // no WorktContext, no export
        if (null == workContext) {

            log.error("*** exportWorks called with null workContext: aborting");

        } else {
            List<Integer> idList = new ArrayList<Integer>();

            WorkDAO workDAO = new DAOFactory().newWorkDAO();

            /*
             * first get the idList of work,
             * all or a sample
             */

            if (workContext.isSampleExport()) {
                if (log.isInfoEnabled()) {
                    log.info("  -- sample export of work: ["
                            + workContext.getSampleFromIndex()
                            + ", "
                            + workContext.getSampleToIndex()
                            + "]");
                }

                // only export a sample size of the entities
                idList = workDAO.getAllIds().subList(
                        workContext.getSampleFromIndex(),
                        workContext.getSampleToIndex());

            } else {
                if (log.isInfoEnabled()) {
                    log.info("  -- export of all work");
                }

                // export all the entities
                idList = workDAO.getAllIds();

            }
            if (log.isInfoEnabled()) {
                log.info("     retrieved idList of size: "
                        + idList.size());
            }

            /*
             * then export the idList of works
             * into single file
             * or multiple files of record count per file
             */

            if (workContext.getRecsPerFile() > 0) {
                // multiple files of recsPerFile

                if (log.isInfoEnabled()) {
                    log.info("    -- exporting muliple files of "
                            + workContext.getRecsPerFile()
                            + " records per file.");
                }

                // name the files
                // with appended two digit zero filled sequence number
                int fileSeqNum = 1;
                // from and to of the sublist range
                int fromIndex = 0;
                int toIndex = 0;

                while (fromIndex < idList.size()) {
                    // more list to process

                    // try for a full sized chunk
                    toIndex = fromIndex + workContext.getRecsPerFile();
                    // if we point off the end of the list
                    if (toIndex > idList.size()) {
                        // only go to the end of the list
                        toIndex = idList.size();
                    }

                    /*
                     * note: List.subList(fromIndex, toIndex)
                     * is fromIndex inclusive and toIndex exclusive
                     */

                    if (log.isInfoEnabled()) {
                        log.info("      -- exporting subList range ["
                                + fromIndex
                                + ", "
                                + toIndex
                                + "]");
                    }

                    // export this range
                    exportTheseWorks(workDAO,
                                     String.format(workContext.getFileStem()
                            + "%02d.xml",
                                                   fileSeqNum++),
                                     idList.subList(fromIndex, toIndex));

                    // and start the next range from the excluded end
                    fromIndex = toIndex;
                } // of while more list to process

                if (log.isInfoEnabled()) {
                    log.info("    ---- done.\n");
                }
            } // of multiple files
            else {
                // one file of all recs

                if (log.isInfoEnabled()) {
                    log.info("    -- exporting one file of "
                            + idList.size()
                            + " work.");
                }

                exportTheseWorks(workDAO,
                                 workContext.getFileStem() + ".xml",
                                 idList);

                if (log.isInfoEnabled()) {
                    log.info("    ---- done.\n");
                }
            }
        }
    }

    /**
     *  Export a set of works.
     *
     * @param workDAO an intitialized workDAO.
     * @param fileName pathed fileName to create.
     * @param workIdList List of Integer work id to export.
     * @throws FileNotFoundException if thrown.
     * @throws UnsupportedEncodingException if thrown.
     * @throws XMLStreamException if thrown.
     */
    private void exportTheseWorks(final WorkDAO workDAO,
                                  final String fileName,
                                  final List<Integer> workIdList)
            throws FileNotFoundException,
                   UnsupportedEncodingException,
                   XMLStreamException {

        // export the idList to the fileName
        // shared OutputStream
        final OutputStream outputStream =
                new FileOutputStream(XMLExportContext.getFileDirPath() + fileName);

        // PrintWriter to OutputStream
        final PrintWriter xmlOut =
                new PrintWriter(new BufferedWriter(new OutputStreamWriter(
                outputStream, "UTF-8")));

        // XMLStreamWriter to OuputStream
        final XMLStreamWriter xmlWrite =
                XMLOutputFactory.newInstance().createXMLStreamWriter(
                outputStream, "UTF-8");

        EfrbrWriter efrbrWriter = new EfrbrWriter();

        WorkWriter workWriter = new WorkWriter();

        if (log.isInfoEnabled()) {
            log.info("        exporting "
                    + workIdList.size()
                    + " work to "
                    + XMLExportContext.getFileDirPath() + fileName);
        }

        // write the document start
        efrbrWriter.writeDocumentStart(xmlOut);

        // write the recordSet start
        efrbrWriter.writeRecordSetStart(xmlOut, null);

        // write the entities start
        efrbrWriter.writeEntitiesStart(xmlOut);

        // write out the idList of Works
        for (Integer workIdInt : workIdList) {

            workWriter.writeEfrbrWork(xmlOut,
                                 xmlWrite,
                                 workDAO.getById(workIdInt));

        }

        // write the entities end
        efrbrWriter.writeEntitiesEnd(xmlOut);

        // write the recordSet end
        efrbrWriter.writeRecordSetEnd(xmlOut);

        xmlOut.flush();
        xmlOut.close();
    }
}
