/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.manifestation.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class ManifestationWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;

    public ManifestationWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEfrbrManifestation(PrintWriter xmlOut,
                                        XMLStreamWriter xmlWrite,
                                        ManifestationJpa manif)
            throws XMLStreamException {

        xmlOut.print(INDENT_2 + "<efrbr-manifestation:manifestation");

        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + XMLExportContext.makeURI("manifestation", manif.getId())
                + "\"");
        xmlOut.println(">");

        xmlOut.println(INDENT_3 + "<!-- titles: [" + manif.getTitles().size() + "] -->");
        for (ManifestationTitle title : manif.getTitles()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:titleOfTheManifestation");
            // TypeOffset
            attribWriter.writeTypeOffset(xmlOut, xmlWrite, INDENT_3, title);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:titleOfTheManifestation>");
        }

        xmlOut.println(INDENT_3 + "<!-- responsibilities: [" + manif.getResponsibilities().size() + "] -->");
        for (ManifestationResponsibility resp : manif.getResponsibilities()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:statementOfResponsibility");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, resp);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:statementOfResponsibility>");
        }

        xmlOut.println(INDENT_3 + "<!-- designations: [" + manif.getDesignations().size() + "] -->");
        for (ManifestationDesignation desig : manif.getDesignations()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:editionIssueDesignation");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, desig);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:editionIssueDesignation>");
        }

        xmlOut.println(INDENT_3 + "<!-- publications: [" + manif.getPublications().size() + "] -->");
        for (ManifestationPublicationJpa pub : manif.getPublications()) {
            xmlOut.println(INDENT_3 + "<efrbr-manifestation:publicationDistribution>");

            if (pub.getPlace() != null) {
                xmlOut.print(INDENT_4 + "<efrbr-manifestation:placeOfPublicationDistribution");
                //  TypeJurisNormalVocab
                attribWriter.writeTypeJurisNormalVocab(xmlOut, xmlWrite, INDENT_4, pub.getPlace());
                xmlOut.println(INDENT_4 + "</efrbr-manifestation:placeOfPublicationDistribution>");
            }

            if (pub.getPublisher() != null) {
                xmlOut.print(INDENT_4 + "<efrbr-manifestation:publisherDistributor");
                // TypeNormalVocab
                attribWriter.writeTypeNormalVocab(xmlOut, xmlWrite, INDENT_4, pub.getPublisher());
                xmlOut.println(INDENT_4 + "</efrbr-manifestation:publisherDistributor>");
            }

            if (pub.getDate() != null) {
                xmlOut.print(INDENT_4 + "<efrbr-manifestation:dateOfPublicationDistribution");
                // TypeNormal
                attribWriter.writeTypeNormal(xmlOut, xmlWrite, INDENT_4, pub.getDate());
                xmlOut.println(INDENT_4 + "</efrbr-manifestation:dateOfPublicationDistribution>");
            }

            xmlOut.println(INDENT_3 + "</efrbr-manifestation:publicationDistribution>");
        }

        xmlOut.println(INDENT_3 + "<!-- fabricators: [" + manif.getFabricators().size() + "] -->");
        for (ManifestationFabricator fab : manif.getFabricators()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:fabricatorManufacturer");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, fab);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:fabricatorManufacturer>");
        }

        xmlOut.println(INDENT_3 + "<!-- seriesStatements: [" + manif.getSeriesStmts().size() + "] -->");
        for (ManifestationSeriesStatement ss : manif.getSeriesStmts()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:seriesStatement");
            // TypeVocab
            attribWriter.writeTypeVocab(xmlOut, xmlWrite, INDENT_3, ss);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:seriesStatement>");
        }

        xmlOut.println(INDENT_3 + "<!-- carrierForms: [" + manif.getCarrierForms().size() + "] -->");
        for (ManifestationCarrierForm form : manif.getCarrierForms()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:formOfCarrier");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, form);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:formOfCarrier>");
        }

        xmlOut.println(INDENT_3 + "<!-- carrierExtents: [" + manif.getCarrierExtents().size() + "] -->");
        for (ManifestationCarrierExtent extent : manif.getCarrierExtents()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:extentOfTheCarrier");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, extent);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:extentOfTheCarrier>");
        }

        xmlOut.println(INDENT_3 + "<!-- physicalMediums: [" + manif.getPhysicalMediums().size() + "] -->");
        for (ManifestationPhysicalMedium medium : manif.getPhysicalMediums()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:physicalMedium");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, medium);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:physicalMedium>");
        }

        xmlOut.println(INDENT_3 + "<!-- captureModes: [" + manif.getCaptureModes().size() + "] -->");
        for (ManifestationCaptureMode mode : manif.getCaptureModes()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:captureMode");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, mode);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:captureMode>");
        }

        xmlOut.println(INDENT_3 + "<!-- carrierDimensions: [" + manif.getCarrierDimensions().size() + "] -->");
        for (ManifestationCarrierDimension dimen : manif.getCarrierDimensions()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:dimensionsOfTheCarrier");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, dimen);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:dimensionsOfTheCarrier>");
        }

        xmlOut.println(INDENT_3 + "<!-- identifiers: [" + manif.getIdentifiers().size() + "] -->");
        for (ManifestationIdentifier ident : manif.getIdentifiers()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:manifestationIdentifier");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, ident);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:manifestationIdentifier>");
        }

        xmlOut.println(INDENT_3 + "<!-- AAASources: [" + manif.getAcqAccAuthSources().size() + "] -->");
        for (ManifestationAAASource src : manif.getAcqAccAuthSources()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:sourceForAcquisitionAccessAuthorization");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, src);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:sourceForAcquisitionAccessAuthorization>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="termsOfAvailability"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

        xmlOut.println(INDENT_3 + "<!-- accessRestrictions:  [" + manif.getAccessRestrictions().size() + "] -->");
        for (ManifestationAccessRestriction restr : manif.getAccessRestrictions()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:accessRestrictionsOnTheManifestation");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, restr);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:accessRestrictionsOnTheManifestation>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="typeface"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="typeSize"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="foliation"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="collation"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="publicationStatus"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="numbering"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

        xmlOut.println(INDENT_3 + "<!-- playingSpeeds: [" + manif.getPlayingSpeeds().size() + "] -->");
        for (ManifestationPlayingSpeed speed : manif.getPlayingSpeeds()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:playingSpeed");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, speed);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:playingSpeed>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="grooveWidth"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="kindOfCutting"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

        xmlOut.println(INDENT_3 + "<!-- tapeConfigurations: [" + manif.getTapeConfigs().size() + "] -->");
        for (ManifestationTapeConfiguration config : manif.getTapeConfigs()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:tapeConfiguration");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, config);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:tapeConfiguration>");
        }

        xmlOut.println(INDENT_3 + "<!-- soundKinds: [" + manif.getSoundKinds().size() + "] -->");
        for (ManifestationSoundKind kind : manif.getSoundKinds()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:kindOfSound");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, kind);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:kindOfSound>");
        }

        xmlOut.println(INDENT_3 + "<!-- reproductionCharacteristics: [" + manif.getReproChars().size() + "] -->");
        for (ManifestationReproductionCharacteristic charc : manif.getReproChars()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:specialReproductionCharacteristic");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, charc);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:specialReproductionCharacteristic>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="colour"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="reductionRatio"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="polarity"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="generation"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="presentationFormat"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="systemRequirements"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

        xmlOut.println(INDENT_3 + "<!-- fileCharacteristics: [" + manif.getFileChars().size() + "] -->");
        for (ManifestationFileCharacteristic charc : manif.getFileChars()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:fileCharacteristics");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, charc);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:fileCharacteristics>");
        }

        xmlOut.println(INDENT_3 + "<!-- accessModes: [" + manif.getAccessModes().size() + "] -->");
        for (ManifestationAccessMode mode : manif.getAccessModes()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:modeOfAccess");
            // Vocab
            attribWriter.writeVocab(xmlOut, xmlWrite, INDENT_3, mode);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:modeOfAccess>");
        }

//            <xsd:element ref="accessAddress"
//                         minOccurs="0" maxOccurs="unbounded"/>
        /*
         * in the 1.0 release, accessAddress is Simple attribute type
         * this is an error, but that is how it is
         */
        xmlOut.println(INDENT_3 + "<!-- accessAddresses: ["
                + manif.getAccessAddresses().size() + "] -->");
        for (ManifestationAccessAddress addr : manif.getAccessAddresses()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:accessAddress");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, addr);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:accessAddress>");
        }

//            <!--
//              extensions to entity attributes
//              -->

        /* ============
         *  insert the formOfExpression into the notes
         */
        xmlOut.println(INDENT_3 + "<!-- notes:[" + (manif.getNotes().size()) + "] -->");
        // insert formOfExpression as the first note
        xmlOut.print(INDENT_3 + "<efrbr-manifestation:note");
        attribWriter.writeString(xmlOut, xmlWrite, INDENT_3,
                                 "formOfExpression: " + manif.getFormOfExpression());
        xmlOut.println(INDENT_3 + "</efrbr-manifestation:note>");
        // insert typeOfScore as the second note, if one
//        if (null != manif.getTypeOfScore()) {
//            xmlOut.print(INDENT_3 + "<efrbr-manifestation:note");
//            attribWriter.writeString(xmlOut, xmlWrite, INDENT_3,
//                                     "typeOfScore: " + manif.getTypeOfScore());
//            xmlOut.println(INDENT_3 + "</efrbr-manifestation:note>");
//        }
        // output the actual notes (if any)
        for (ManifestationNote note : manif.getNotes()) {
            xmlOut.print(INDENT_3 + "<efrbr-manifestation:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);
            xmlOut.println(INDENT_3 + "</efrbr-manifestation:note>");
        }

        xmlOut.println(INDENT_2 + "</efrbr-manifestation:manifestation>");

    }
}
